
import math

def ra_to_degrees(text): # 00:05:09.90
	ra = text.split(":") # ["00", "05", "09.90"]
	hours, minutes, seconds = ra
	
	hours = float(hours)
	minutes = float(minutes)
	seconds = float(seconds)
	
	degrees = (hours / 24.0 * 360.0) + (minutes / (24.0 * 60.0) * 360.0) + (seconds / (24.0 * 60.0 * 60.0) * 360.0)
	return degrees
	
def dec_to_degrees(text): # 00:05:09.90
	dec = text.split(":") # ["00", "05", "09.90"]
	deg, minutes, seconds = dec
	
	deg = float(deg)
	minutes = float(minutes)
	seconds = float(seconds)
	
	degrees = deg + (minutes / 60) + (seconds / (60 * 60))
	return degrees
	
def base_radius_log(mag, distance, reference_length, radius_of_polaris, min_radius): #logarithmic relationship
	m = mag
	d = distance
	l = reference_length
	p = radius_of_polaris
	s = min_radius
	pa = math.pi * p * p
	sa = math.pi * s * s
	t = math.pow(pa / sa, 2.0 / 9.0)
	a = pa * t
	q = a / math.pow(t, m - 1.0)
	r = math.sqrt(q / math.pi)
	final_radius = r * distance / reference_length
	return final_radius
	
def base_radius_k(mag, distance, reference_length, radius_of_polaris, min_radius): #linear relationship
	m = mag
	d = distance
	l = reference_length
	pol = radius_of_polaris
	min = min_radius
	slope = (min - pol) / 4.5
	y_int = pol - (4 / 9) * (min - pol)
	r = slope * mag + y_int
	final_radius = r * distance / reference_length
	return final_radius
	
def coorprime(ra_rad, dec_rad, dec_rotation): # remember arg3 takes degrees
	ra = ra_rad
	dec = dec_rad
	r = math.radians(dec_rotation)
	opp = math.sin(dec)
	adj = math.cos(dec) * math.cos(ra)
	beta = math.atan(opp / adj)
	if math.cos(ra) > 0:
		beta_p = beta - r
	if math.cos(ra) < 0:
		beta_p = r - beta
	v = math.sqrt(opp * opp + adj * adj)
	w = v * math.sin(beta_p)
	u = math.cos(dec) * math.sin(ra)
	dec_p = math.asin(w)
	z = math.cos(dec_p)
	ra_p = math.asin(u / z)
	if math.cos(ra) < 0:
		if beta_p < math.pi / 2:
			ra_p = math.pi - ra_p
		# if beta_p > pi/2 then ra_p stays the same
	if math.cos(ra) > 0:
		if beta_p < -math.pi / 2:
			ra_p = math.pi - ra_p
	return ra_p, dec_p	

def hex(n):
	if n == 10:
		n = "A"
	if n == 11:
		n = "B"
	if n == 12:
		n = "C"
	if n == 13:
		n = "D"
	if n == 14:
		n = "E"
	if n == 15:
		n = "F"
	if n < 10:
		n = str(int(n))	
	return n
	
def color(r, g, b):
	r = float(r)
	g = float(g)
	b = float(b)
	
	r = r / 16
	g = g / 16
	b = b / 16
	
	r1 = math.floor(r)
	r2 = 16 * (r - math.floor(r))
	g1 = math.floor(g)
	g2 = 16 * (g - math.floor(g))
	b1 = math.floor(b)
	b2 = 16 * (b - math.floor(b))
	
	r1 = hex(r1)
	r2 = hex(r2)
	g1 = hex(g1)
	g2 = hex(g2)
	b1 = hex(b1)
	b2 = hex(b2)
	
	fill = "#%s%s%s%s%s%s" % (r1, r2, g1, g2, b1, b2)
	return fill
	
def ellipse(index, x, y, x_radius, y_radius, rotation, stroke_width, fill):
	return '%d <ellipse transform="translate(%f %f) rotate(%f)" ry="%f" rx="%f" stroke-width="%s" stroke="%s" fill="%s"/> \n' % (index, x, y, rotation, y_radius, x_radius, stroke_width, fill, fill)	
	
def get_parameters(input):
	if input == "D":
		filename = "coorlist.txt"
		e = 450
		f = 450
		g = 450
		h = 450
		i = 450
		j = 450
		ra_rotation = 10 
		dec_rotation = 10
		radius_of_polaris = 5 
		min_radius = 2
		lim_mag = 5
		kind =   "k" 
		stroke_width = 1
		fill =  "#ffffff" 
		s = 1
		allowed_domain = 5
		
	if input == "F":
		parameters = raw_input("Which parameter file? > ")
		stuff = open(parameters).read().split("\n")
		
		filename = stuff[0].split()[2]
		e = float(stuff[1].split()[2])
		f = float(stuff[2].split()[2])
		g = float(stuff[3].split()[2])
		h = float(stuff[4].split()[2])
		i = float(stuff[5].split()[2])
		j = float(stuff[6].split()[2])
		ra_rotation = float(stuff[7].split()[2])
		dec_rotation = float(stuff[8].split()[2])
		radius_of_polaris = float(stuff[9].split()[2])
		min_radius = float(stuff[10].split()[2])
		lim_mag = float(stuff[11].split()[2])
		kind = stuff[12].split()[2] 
		stroke_width = float(stuff[13].split()[2])
		fill = stuff[14].split()[2]
		s = float(stuff[15].split()[2])
		allowed_domain = float(stuff[16].split()[2])

	if input == "IP":
		filename = raw_input("Which star catalogue file would you like to use? > ")

		print "What are your parameters?"

		e = float(raw_input("e? > "))
		f = float(raw_input("f? > "))
		g = float(raw_input("g? > "))
		h = float(raw_input("h? > "))
		i = float(raw_input("i? > "))
		j = float(raw_input("j? > "))

		ra_rotation = float(raw_input("Right ascension rotation? (Counterclockwise from above) > "))

		dec_rotation = float(raw_input("Declination rotation? (Counterclockwise when g is at 3 o'clock) > "))

		radius_of_polaris = float(raw_input("What is the set radius for Polaris? > "))

		min_radius = float(raw_input("What is the set radius for a star with magnitude of 6.5 > "))

		lim_mag = float(raw_input("Limiting magnitude? > "))

		kind = raw_input("Logarithmic or linear? (log or k) > ")
	
		stroke_width = raw_input("Stroke width? > ")
		fill = raw_input("Any fill? > ")
		
		if fill == "yes":
			mu = raw_input("Any color? > ")
			if mu == "no":
				fill = "#000000"
			if mu == "yes":
				nu = raw_input("Enter a hexadecimal or RGB values? (hex or RGB) > ")
			if nu == "hex":
				hex = "Enter hexadecimal. > "
			if nu == "RGB":
				print "Enter values from 0 to 255."
				red = raw_input("How much red? > ")
				green = raw_input("How much green? > ")
				blue = raw_input("How much blue? > ")	
					
				fill = color(red, green , blue)
	
		s = float(raw_input("What is the tolerance (in percent)? > "))

		allowed_domain = int(raw_input("Which domain? > "))
	
	return filename, e, f, g, h, i, j, ra_rotation, dec_rotation, radius_of_polaris, min_radius, lim_mag, kind, stroke_width, fill, s, allowed_domain


##### start process


def process():
	global filename, e, f, g, h, i, j, ra_rotation, dec_rotation, radius_of_polaris, min_radius, lim_mag, kind, stroke_width, fill, s, allowed_domain
	
	cross_distance = math.sqrt((e + g) * (e + g) + (f + h) * (f + h) + (i + j) * (i + j))
	s = s * 0.01 * cross_distance


	if allowed_domain == 1:
		k = f
		l = h
		m = i
		n = j
		t = e

	if allowed_domain == 2:
		k = g
		l = e
		m = i
		n = j
		t = f

	if allowed_domain == 3:
		k = h
		l = f
		m = i
		n = j
		t = g

	if allowed_domain == 4:
		k = e
		l = g
		m = i
		n = j
		t = h

	if allowed_domain == 5:
		k = f
		l = h
		m = g
		n = e
		t = i	

	width = k + l
	height = m + n

	part1 = '<html>\n<head>\n<title>Night Sky</title>\n</head>\n<body bgcolor="black">\n<svg width="%d" height="%d" xmlns="http://www.w3.org/2000/svg"> \n' % (width, height)

	list = open(filename).read().split("\n")
	
	part2 = ""
	
	for line in list:	
	
		spaces_list = line.split()

		index = spaces_list[0]
		index = int(index)

		ra_hms = spaces_list[1]
		ra_deg = ra_to_degrees(ra_hms)

		dec_dms = spaces_list[2]
		dec_deg = dec_to_degrees(dec_dms)

		mag = spaces_list[6]
		if mag == "????":
			continue	
		else:
			mag = float(mag)
		if mag > lim_mag:
			continue	

		ra_deg = ra_deg + ra_rotation

		# dec_rotation

		ra_rad = math.radians(ra_deg)
		dec_rad = math.radians(dec_deg)

		prime_list = coorprime(ra_rad, dec_rad, dec_rotation)
		ra_rad = prime_list[0]
		dec_rad = prime_list[1]

		if allowed_domain == 1:
			if math.cos(ra_rad) < 0:
				continue

		if allowed_domain == 2:
			if math.sin(ra_rad) < 0:
				continue
			ra_rad = ra_rad - math.pi / 2

		if allowed_domain == 3:
			if math.cos(ra_rad) > 0:
				continue
			ra_rad = ra_rad - math.pi

		if allowed_domain == 4:
			if math.sin(ra_rad) > 0:
				continue
			ra_rad = ra_rad - 3 * math.pi / 2

		if allowed_domain == 5:
			if math.sin(dec_rad) < 0:
				continue
			prime_list = coorprime(ra_rad, dec_rad, 90)
			ra_rad = prime_list[0]
			dec_rad = prime_list[1]

		p = math.tan(ra_rad) * t
		v = t / math.cos(ra_rad)
		q = math.tan(dec_rad) * v

		x = k - p
		y = m - q	

		if x < -s or x > width + s:
			continue

		if y < -s or y > height + s:
			continue		

		distance = math.sqrt(t * t + p * p + q * q)
		gamma = math.acos(t / distance)

		if kind == "log":
			final_radius = base_radius_log(mag, distance, t, radius_of_polaris, min_radius)
		if kind == "k":
			final_radius = base_radius_k(mag, distance, t, radius_of_polaris,  min_radius)
		x_radius = final_radius / math.cos(gamma)
		y_radius = final_radius

		theta = math.atan(q / p)
		theta = math.degrees(theta)

		part2 = part2 + ellipse(index, x, y, x_radius, y_radius,
			      theta, stroke_width, fill) 	
	part3 = "</svg>\n</body>\n</html>"
	
	return part1 + part2 + part3

##### end process

target_file = open(raw_input("Which file would you like to write to? > "), 'w')

result = raw_input("Run default parameters, run a parameters file, or input parameters? (D, F, or IP) > ")

var = get_parameters(result)
filename = var[0]
e = var[1]
f = var[2]
g = var[3]
h = var[4]
i = var[5]
j = var[6]
ra_rotation = var[7]
dec_rotation = var[8]
radius_of_polaris = var[9]
min_radius = var[10]
lim_mag = var[11]
kind = var[12]
stroke_width = var[13]
fill = var[14]
s = var[15]
allowed_domain = var[16]

coordinates = process()

coordinates = str(coordinates)

target_file.write(coordinates)

target_file.close()


	
	
	
	
	
	
	
	